@@ -50,7 +50,6 @@ import apt_pkg
 import os
 import re
 import logging
-import time
 import threading
 
 from gettext import gettext as _
@@ -198,14 +197,9 @@ class CellAreaPackage(Gtk.CellAreaBox):
         if not self.column:
             return 0
         else:
-            val = GObject.Value()
-            val.init(int)
-            widget.style_get_property("horizontal-separator", val)
-            h_sep = val.get_int()
-            widget.style_get_property("grid-line-width", val)
-            line_width = val.get_int()
-            cell_start = self.column.get_x_offset() - h_sep - line_width
+            cell_start = self.column.get_x_offset()
             if not self.indent_toplevel:  # i.e. if no headers
+                val = GObject.Value()
                 widget.style_get_property("expander-size", val)
                 spacing = self.get_property("spacing")
                 # Hardcode 4 because GTK+ hardcodes 4 internally
@@ -660,12 +654,29 @@ class UpdatesAvailable(InternalDialog):
                     "changelog information."
                 )
             )
-        # else, get it from the entwork
+            return
+        # else, get it from the network
         elif self.expander_details.get_expanded():
-            lock = threading.Lock()
-            lock.acquire()
+            button = Gtk.Button(stock="gtk-cancel")
+            callback_args = (path, widget, name, changes_buffer, long_desc)
+            id = button.connect("clicked", lambda _, args: hide_button(args),
+                                callback_args)
+
+            def hide_button(args):
+                nonlocal id, button
+                if id is not None:
+                    button.disconnect(id)
+                    id = None
+                    button.set_visible(0)
+                    self.display_changes(args)
+
+            def on_thread_done(args):
+                GLib.idle_add(hide_button, args)
+
             changelog_thread = threading.Thread(
-                target=self.cache.get_news_and_changelog, args=(name, lock)
+                target=self.cache.get_news_and_changelog,
+                args=(name,),
+                kwargs={'cb': on_thread_done, 'cb_args': callback_args}
             )
             changelog_thread.start()
             changes_buffer.set_text(
@@ -673,21 +684,14 @@ class UpdatesAvailable(InternalDialog):
             )
             iter = changes_buffer.get_iter_at_line(1)
             anchor = changes_buffer.create_child_anchor(iter)
-            button = Gtk.Button(stock="gtk-cancel")
             self.textview_changes.add_child_at_anchor(button, anchor)
             button.show()
-            id = button.connect(
-                "clicked", lambda w, lock: lock.release(), lock
-            )
-            # wait for the dl-thread
-            while lock.locked():
-                time.sleep(0.01)
-                while Gtk.events_pending():
-                    Gtk.main_iteration()
-            # download finished (or canceld, or time-out)
-            button.hide()
-            if button.handler_is_connected(id):
-                button.disconnect(id)
+            return
+        args_normal = (path, widget, name, changes_buffer, long_desc)
+        self.display_changes(args_normal)
+
+    def display_changes(self, args):
+        (path, widget, name, changes_buffer, long_desc) = args
         # check if we still are in the right pkg (the download may have taken
         # some time and the user may have clicked on a new pkg)
         now_path = widget.get_cursor()[0]
@@ -783,8 +787,6 @@ class UpdatesAvailable(InternalDialog):
             )
         else:
             self.window_main.get_window().set_cursor(None)
-        while Gtk.events_pending():
-            Gtk.main_iteration()
 
     def _mark_selected_updates(self):
         def foreach_cb(model, path, iter, data):
@@ -821,6 +823,7 @@ class UpdatesAvailable(InternalDialog):
         self.hbox_restart.set_visible(requires_restart)
 
     def _refresh_updates_count(self):
+        self.button_install.show()
         self.button_install.set_sensitive(
             self.cache.install_count + self.cache.del_count
         )
@@ -856,7 +859,7 @@ class UpdatesAvailable(InternalDialog):
                       and not self.list.ubuntu_pro_groups):
                     download_str = _("You need to enable Ubuntu Pro to install"
                                      " these updates.")
-                    self.button_install.destroy()
+                    self.button_install.hide()
                 else:
                     download_str = _("There are no updates to install.")
                     self.button_install.set_sensitive(False)
@@ -1255,7 +1258,5 @@ class UpdatesAvailable(InternalDialog):
         self.update_close_button()
         self.update_count()
         self.setBusy(False)
-        while Gtk.events_pending():
-            Gtk.main_iteration()
         self.updates_changed()
         return False
